@@ -1,6 +1,7 @@
 import asyncio
 import sys
 import traceback
+from collections import UserList
 from typing import Callable, Dict, Iterable, List, Optional, Type, Union
 
 from .builtin.event import ExceptionThrowed
@@ -10,26 +11,29 @@ from .entities.exectarget import ExecTarget
 from .entities.listener import Listener
 from .entities.namespace import Namespace
 from .entities.signatures import Force, RemoveMe
-from .exceptions import (
-    DisabledNamespace,
-    ExecutionStop,
-    ExistedNamespace,
-    InvalidEventName,
-    PropagationCancelled,
-    RegisteredEventListener,
-    RequirementCrashed,
-    UnexistedNamespace,
-)
+from .exceptions import (DisabledNamespace, ExecutionStop, ExistedNamespace,
+                         InvalidEventName, PropagationCancelled,
+                         RegisteredEventListener, RequirementCrashed,
+                         UnexistedNamespace)
 from .interfaces.decorator import DecoratorInterface
 from .interfaces.dispatcher import DispatcherInterface
 from .typing import T_Dispatcher
-from .utilles import (
-    Ctx,
-    argument_signature,
-    dispatcher_mixin_handler,
-    group_dict,
-    run_always_await_safely,
-)
+from .utilles import (Ctx, argument_signature, dispatcher_mixin_handler,
+                      group_dict, run_always_await_safely)
+
+
+class DebugList(UserList):
+    def extend(self, item) -> None:
+        print(item)
+        return super().extend(item)
+
+    def append(self, item) -> None:
+        print(item)
+        return super().append(item)
+
+    def insert(self, i: int, item) -> None:
+        print(i, item)
+        return super().insert(i, item)
 
 
 class Broadcast:
@@ -87,19 +91,12 @@ class Broadcast:
             )
         )
 
-    async def layered_scheduler(
-        self, listener_generator: Iterable[Listener], event: Dispatchable
-    ):
-        grouped: Dict[int, List[Listener]] = group_dict(
-            listener_generator, lambda x: x.priority
-        )
+    async def layered_scheduler(self, listener_generator: Iterable[Listener], event: Dispatchable):
+        grouped: Dict[int, List[Listener]] = group_dict(listener_generator, lambda x: x.priority)
         event_dispatcher_mixin = dispatcher_mixin_handler(event.Dispatcher)
         with self.event_ctx.use(event):
             for _, current_group in sorted(grouped.items(), key=lambda x: x[0]):
-                coros = [
-                    self.Executor(target=i, dispatchers=event_dispatcher_mixin)
-                    for i in current_group
-                ]
+                coros = [self.Executor(target=i, dispatchers=event_dispatcher_mixin) for i in current_group]
                 done_tasks, _ = await asyncio.wait(coros)
                 for task in done_tasks:
                     if task.exception().__class__ is PropagationCancelled:
@@ -120,34 +117,38 @@ class Broadcast:
             is_exectarget = True
 
         if is_listener:
-            if target.namespace.disabled:
+            if target.namespace.disabled:  # type: ignore
                 raise DisabledNamespace(
-                    "catched a disabled namespace: {0}".format(target.namespace.name)
+                    "caught a disabled namespace: {0}".format(target.namespace.name)  # type: ignore
                 )
 
-        target_callable = target.callable if is_exectarget else target
+        target_callable = target.callable if is_exectarget else target  # type: ignore
         parameter_compile_result = {}
 
         dii = self.dispatcher_interface
 
-        dispatchers = dispatchers or []
+        dispatchers = dispatchers and dispatchers.copy() or []
+
         if is_exectarget:
-            dispatchers.extend(target.inline_dispatchers)
+            dispatchers.extend(target.dispatchers)
             if is_listener:
-                dispatchers.extend(target.namespace.injected_dispatchers)
+                dispatchers.extend(target.namespace.injected_dispatchers)  # type: ignore
 
         if use_global_dispatchers:
-            dispatchers.extend(self.global_dispatchers)
+            for i in self.global_dispatchers:
+                dispatchers.insert(0, i)
 
         dii.start_execution(dispatchers)
         try:
-            await dii.exec_lifecycle("beforeExecution")
+            for dispatcher in dii.execution_contexts[-1].dispatchers:
+                i = getattr(dispatcher, "beforeExecution", None)
+                if i:
+                    await run_always_await_safely(i, dii)  # type: ignore
+
             if is_exectarget:
-                for name, annotation, default in argument_signature(target_callable):
-                    oplog = [target.param_paths.setdefault(name, []), 0]
-                    parameter_compile_result[name] = await dii.lookup_param(
-                        name, annotation, default, oplog
-                    )
+                for name, annotation, default in argument_signature(target_callable):  # type: ignore
+                    optimized_log = target.param_paths.setdefault(name, [])
+                    parameter_compile_result[name] = await dii.lookup_param(name, annotation, default, optimized_log)
 
                 for hl_d in target.decorators:
                     await dii.lookup_by_directly(
@@ -158,15 +159,15 @@ class Broadcast:
                     )
 
             else:
-                for name, annotation, default in argument_signature(target_callable):
-                    parameter_compile_result[name] = await dii.lookup_param(
-                        name, annotation, default, [[], 0]
-                    )
+                for name, annotation, default in argument_signature(target_callable):  # type: ignore
+                    parameter_compile_result[name] = await dii.lookup_param(name, annotation, default, [])
 
-            await dii.exec_lifecycle("afterDispatch", None, None)
-            result = await run_always_await_safely(
-                target_callable, **parameter_compile_result
-            )
+            for dispatcher in dii.execution_contexts[-1].dispatchers:
+                i = getattr(dispatcher, "afterDispatch", None)
+                if i:
+                    await run_always_await_safely(i, dii, None, None)  # type: ignore
+
+            result = await run_always_await_safely(target_callable, **parameter_compile_result)
         except (ExecutionStop, PropagationCancelled):
             raise
         except RequirementCrashed:
@@ -177,12 +178,16 @@ class Broadcast:
             if event is not None and event.__class__ is not ExceptionThrowed:
                 if print_exception:
                     traceback.print_exc()
-                elif post_exception_event:
+                if post_exception_event:
                     self.postEvent(ExceptionThrowed(exception=e, event=event))
             raise
         finally:
             _, exception, tb = sys.exc_info()
-            await dii.exec_lifecycle("afterExecution", exception, tb)
+            for dispatcher in dii.execution_contexts[-1].dispatchers:
+                i = getattr(dispatcher, "afterExecution", None)
+                if i:
+                    await run_always_await_safely(i, dii, exception, tb)  # type: ignore
+
             dii.clean()
 
         if result.__class__ is Force:
@@ -216,14 +221,10 @@ class Broadcast:
     def getDefaultNamespace(self):
         return self.default_namespace
 
-    def createNamespace(
-        self, name, *, priority: int = 0, hide: bool = False, disabled: bool = False
-    ):
+    def createNamespace(self, name, *, priority: int = 0, hide: bool = False, disabled: bool = False):
         if self.containNamespace(name):
             raise ExistedNamespace(name, "has been created!")
-        self.namespaces.append(
-            Namespace(name=name, priority=priority, hide=hide, disabled=disabled)
-        )
+        self.namespaces.append(Namespace(name=name, priority=priority, hide=hide, disabled=disabled))
         return self.namespaces[-1]
 
     def removeNamespace(self, name):
@@ -291,9 +292,9 @@ class Broadcast:
     ):
         if isinstance(event, str):
             _name = event
-            event = self.findEvent(event)
+            event = self.findEvent(event)  # type: ignore
             if not event:
-                raise InvalidEventName(_name + " is not vaild!")  # type: ignore
+                raise InvalidEventName(_name + " is not valid!")  # type: ignore
         priority = int(priority)
 
         def receiver_wrapper(callable_target):
@@ -305,17 +306,15 @@ class Broadcast:
                         namespace=namespace or self.getDefaultNamespace(),
                         inline_dispatchers=dispatchers,
                         priority=priority,
-                        listening_events=[event],
+                        listening_events=[event],  # type: ignore
                         decorators=decorators,
                     )
                 )
             else:
                 if event not in may_listener.listening_events:
-                    may_listener.listening_events.append(event)
+                    may_listener.listening_events.append(event)  # type: ignore
                 else:
-                    raise RegisteredEventListener(
-                        event.__name__, "has been registered!"
-                    )
+                    raise RegisteredEventListener(event.__name__, "has been registered!")  # type: ignore
             return callable_target
 
         return receiver_wrapper
